/* 
 * Copyright (c) 2010, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/*
 *  ======== LoggerSysTID.c ========
 *  Just like LoggerSys, but also prints thread ID.
 */
#include <xdc/std.h>

#include <xdc/runtime/Diags.h>
#include <xdc/runtime/Log.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Text.h>
#include <xdc/runtime/Types.h>

#include <pthread.h>

#include "package/internal/LoggerSysTID.xdc.h"

#if (xdc_target__bitsPerChar * xdc_target__sizeof_Int) == 32
#define SERIAL "#%010u "
#define HI "%0.8x:"
#define LO "%0.8x] "
#elif (xdc_target__bitsPerChar * xdc_target__sizeof_Long) == 32
#define SERIAL "#%010lu "
#define HI "%0.8lx:"
#define LO "%0.8lx] "
#else
#error xdc.runtime.Log does not support this target.
#endif


static Void doPrint(Log_EventRec *er);

/*
 *  ======== Instance_init =========
 */
Void LoggerSysTID_Instance_init(LoggerSysTID_Object *obj,
    const LoggerSysTID_Params *prms)
{
    obj->enabled = TRUE;
}

/*
 *  ======== LoggerSysTID_disable ========
 */
Bool LoggerSysTID_disable(LoggerSysTID_Object *obj)
{
    Bool prev = obj->enabled;
    obj->enabled = FALSE;
    return (prev);
}

/*
 *  ======== LoggerSysTID_enable ========
 */
Bool LoggerSysTID_enable(LoggerSysTID_Object *obj)
{
    Bool prev = obj->enabled;
    obj->enabled = TRUE;
    return (prev);
}

/*
 *  ======== write0 =========
 */
Void LoggerSysTID_write0(LoggerSysTID_Object *obj, Log_Event evt,
                      Types_ModuleId mid)
{
    LoggerSysTID_write8(obj, evt, mid, 0, 0, 0, 0, 0, 0, 0, 0);
}

/*
 *  ======== write1 =========
 */
Void LoggerSysTID_write1(LoggerSysTID_Object *obj, Log_Event evt,
                      Types_ModuleId mid, IArg a1)
{
    LoggerSysTID_write8(obj, evt, mid, a1, 0, 0, 0, 0, 0, 0, 0);
}

/*
 *  ======== write2 =========
 */
Void LoggerSysTID_write2(LoggerSysTID_Object *obj, Log_Event evt,
                      Types_ModuleId mid, IArg a1, IArg a2)
{
    LoggerSysTID_write8(obj, evt, mid, a1, a2, 0, 0, 0, 0, 0, 0);
}

/*
 *  ======== write4 =========
 */
Void LoggerSysTID_write4(LoggerSysTID_Object *obj, Log_Event evt,
                      Types_ModuleId mid, IArg a1, IArg a2, IArg a3, IArg a4)
{
    LoggerSysTID_write8(obj, evt, mid, a1, a2, a3, a4, 0, 0, 0, 0);
}

/*
 *  ======== write8 =========
 */
Void LoggerSysTID_write8(LoggerSysTID_Object *obj, Log_Event evt,
                      Types_ModuleId mid, IArg a1, IArg a2, IArg a3, IArg a4,
                      IArg a5, IArg a6, IArg a7, IArg a8)
{
    Log_EventRec evrec;

    if (!obj->enabled) {
        return;
    }

    LoggerSysTID_TimestampProxy_get64(&evrec.tstamp);

    evrec.serial = 0;
    evrec.evt = Types_makeEvent(Log_getEventId(evt), mid);
    evrec.arg[0] = a1;
    evrec.arg[1] = a2;
    evrec.arg[2] = a3;
    evrec.arg[3] = a4;
    evrec.arg[4] = a5;
    evrec.arg[5] = a6;
    evrec.arg[6] = a7;
    evrec.arg[7] = a8;

    doPrint(&evrec);
}


/*
 *  ======== doPrint ========
 */
static Void doPrint(Log_EventRec *er)
{
    Text_RopeId rope;
    String      fmt;
    Bits32      hi, lo;
    static Bool busy = FALSE;

    if (!busy) {
        busy = TRUE;

        /*
         *  print serial number if there is one;
         *  0 isn't a valid serial number
         */
        if (er->serial) {
            System_printf(SERIAL, er->serial);
        }

        /* print timestamp if there is one; ~0 isn't a valid timestamp value */
        hi = er->tstamp.hi;
        lo = er->tstamp.lo;
        if (lo != ~0 && hi != ~0) {
            System_printf("[t=0x");
            if (hi) {
                System_printf(HI, hi);
            }
            System_printf(LO, lo);
        }

        System_printf("[tid=0x%x] ", pthread_self());

        /* print module name */
        Text_putMod((Text_RopeId)Types_getModuleId(er->evt), NULL, -1);
        System_printf(": ");

        /* print event */
        rope = Types_getEventId(er->evt);  /* event id is the message rope */
        if (rope == 0) {
            /* Log_print() event */
            System_aprintf((String)iargToPtr(er->arg[0]),
                    er->arg[1], er->arg[2], er->arg[3], er->arg[4],
                    er->arg[5], er->arg[6], 0,          0);
        }
        else {
            /* Log_write() event */
            fmt = Text_ropeText(rope);

            if (Text_isLoaded) {
                System_aprintf(fmt, er->arg[0], er->arg[1], er->arg[2],
                        er->arg[3], er->arg[4], er->arg[5], er->arg[6],
                        er->arg[7]);
            }
            else {
                System_aprintf("{evt: fmt=%p, args=[0x%x, 0x%x ...]}",
                        fmt, er->arg[0], er->arg[1]);
            }
        }

        System_putch('\n');

        busy = FALSE;
    }
}
/*
 *  @(#) ti.sdo.utils.loggers; 1, 0, 0,63; 10-29-2010 17:24:29; /db/atree/library/trees/osal/osal-c12x/src/ xlibrary

 */

